import json
import re
from typing import List

from pygments import highlight
from pygments.formatters.terminal import TerminalFormatter
from pygments.lexers.javascript import JavascriptLexer
from pygments.lexers.text import TexLexer
from pygments.lexers.shell import BashLexer
from requests import Response, request

from ncurl.api import parse_context, ParsedContext


def curl_command_to_context(command: List[str]) -> ParsedContext:
    return parse_context(command)


def request_by_context(context: ParsedContext) -> Response:
    """
    change curl to request
    :param context:
    :return: Request
    """
    data = context.data.encode('utf-8') if context.data else None
    return request(method=context.method, url=context.url, headers=context.headers, data=data,
                   cookies=dict(context.cookies), verify=context.verify)


def response_to_head_str(response: Response) -> str:
    """
    response status and header to string
    :param response:
    :return:
    """
    head_str = ''
    head_str += f'HTTP {response.status_code} {response.reason}\n'
    for key, value in response.headers.items():
        head_str += f"{key}: {value}\n"
    return head_str


def print_response(response: Response) -> None:
    head_str = response_to_head_str(response)
    headers_result = highlight(head_str, BashLexer(), TerminalFormatter())
    print(headers_result)

    try:
        result = highlight(json.dumps(response.json(), indent=4, sort_keys=True, ensure_ascii=False), JavascriptLexer(),
                           TerminalFormatter())
    except ValueError as e:
        result = highlight(response.content.decode("utf-8"), TexLexer(), TerminalFormatter())
    print(result)


def join(split_command):
    """Return a shell-escaped string from *split_command*."""
    return ' '.join(quote(arg) for arg in split_command)


_find_unsafe = re.compile(r'[^\w@%+=:,./-]', re.ASCII).search


def quote(s):
    """Return a shell-escaped version of the string *s*."""
    if not s:
        return "''"
    if _find_unsafe(s) is None:
        return s

    # use single quotes, and put single quotes into double quotes
    # the string $'b is then quoted as '$'"'"'b'
    return "'" + s.replace("'", "'\"'\"'") + "'"



from collections import namedtuple
from pathlib import Path
import json

Config = namedtuple('Config', ['server_url', 'expiredAt', 'isUpload'])
_default_config = Config(
    server_url='https://api.ncurl.sh/api',
    expiredAt=1800,  # default expired at half hour
    isUpload=True
)


def load_or_init_config():
    config_path = Path.joinpath(Path.home(), Path('.ncurl'))
    if not config_path.exists():
        with open(str(config_path), 'w') as f:
            f.write(json.dumps(_default_config._asdict(), indent="  "))
            return _default_config

    with open(str(config_path)) as f:
        return json.loads(f.read(), object_hook=lambda json_data: Config(**json_data))


import argparse
import re
from collections import OrderedDict, namedtuple

from six.moves import http_cookies as Cookie

parser = argparse.ArgumentParser()
parser.add_argument('command')
parser.add_argument('url')
parser.add_argument('-d', '--data')
parser.add_argument('-b', '--data-binary', default=None)
parser.add_argument('-X', '--request', default='')
parser.add_argument('-H', '--header', action='append', default=[])
parser.add_argument('--compressed', action='store_true')
parser.add_argument('--insecure', action='store_true')
parser.add_argument('-L', '--location', action='store_true')

BASE_INDENT = " " * 4

ParsedContext = namedtuple('ParsedContext', ['method', 'url', 'data', 'headers', 'cookies', 'verify'])


def parse_context(tokens):
    method = "get"

    parsed_args = parser.parse_args(tokens)

    post_data = parsed_args.data or parsed_args.data_binary
    if post_data:
        method = 'post'

    if 'X' in parsed_args:
        method = parsed_args.X.lower()

    cookie_dict = OrderedDict()
    quoted_headers = OrderedDict()

    for curl_header in parsed_args.header:
        if curl_header.startswith(':'):
            occurrence = [m.start() for m in re.finditer(':', curl_header)]
            header_key, header_value = curl_header[:occurrence[1]], curl_header[occurrence[1] + 1:]
        else:
            header_key, header_value = curl_header.split(":", 1)

        if header_key.lower() == 'cookie':
            cookie = Cookie.SimpleCookie(header_value)
            for key in cookie:
                cookie_dict[key] = cookie[key].value
        else:
            quoted_headers[header_key] = header_value.strip()

    return ParsedContext(
        method=method,
        url=parsed_args.url,
        data=post_data,
        headers=quoted_headers,
        cookies=cookie_dict,
        verify=parsed_args.insecure
    )



#!/usr/bin/env python

import subprocess
import sys
import requests
import colorama

from ncurl.conf import load_or_init_config, Config
from ncurl.utils import join
from ncurl.curl_utils import CurlUtils


def do_curl(config: Config):
    command = ['curl', '-sS'] + sys.argv[1:]

    process = subprocess.Popen(command,
                               stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE)
    stdout, stderr = process.communicate()
    curl_utils = CurlUtils(command=command, output=stdout.decode('utf-8'), stderr=stderr.decode('utf-8'))
    curl_utils.highlight()

    if not config.isUpload:
        return

    upload_contents = list(map(lambda content: dict(content=content.content, highlightName=content.lexer.name.lower()),
                               curl_utils.contents))
    result = requests.post(f'{config.server_url}/instants', json={
        "commands": join(['curl'] + sys.argv[1:]),
        "contents": upload_contents,
        "expire": config.expiredAt
    })
    response = result.json()
    colorama.init()
    if result.status_code == 200:
        print(f'\n {colorama.Fore.GREEN}View and share in: {response.get("webUrl")}')
    else:
        print(f'\n {colorama.Fore.RED}Failed upload to server: {response.content.decode("utf-8")}')
        sys.exit(1)


def main():
    config = load_or_init_config()
    do_curl(config)


if __name__ == '__main__':
    main()




import re
from json.decoder import JSONDecodeError
from typing import List

from pygments.formatters.terminal import TerminalFormatter
from pygments.lexer import RegexLexer
from pygments.lexers import guess_lexer
import json

from pygments.lexers.data import JsonLexer
from pygments import highlight

import logging

from pygments.lexers.shell import BashLexer

logger = logging.getLogger(__name__)


class OutputContent(object):

    def __init__(self, lexer: RegexLexer, content: str):
        self.lexer = lexer
        self.content = content

    def __str__(self) -> str:
        return f"content: {self.content}, lexer: {self.lexer}"

    def __repr__(self) -> str:
        return self.__str__()


class CurlUtils(object):
    contents: List[OutputContent] = None
    include: bool = False
    verbose: bool = False

    def __init__(self, command: List[str], output: str, stderr: str = ''):
        self._output = output
        self._stderr = stderr
        self._command = command
        self.contents = []
        self.command_line_args_parse()
        self.parse_output()

    def command_line_args_parse(self):
        command = self._command
        self.include = '-i' in command or '--include' in command
        self.verbose = '-v' in command or '--verbose' in command

    def _verbose_parse_output(self):
        stderr_lines = self._stderr.splitlines()
        lines = []
        for index, line in enumerate(stderr_lines):
            if not re.match(r"^{|} \[\d+ bytes data\]", line):
                lines.append(line)

        for index, line in enumerate(lines):
            if re.match(r"^<$", line.strip()):
                header_content = '\n'.join(lines[:index+1])
                self.contents.append(OutputContent(BashLexer(), header_content))
                for sub_index, sub_line in enumerate(lines[index+1:], start=index+1):
                    if re.match("^\* Connection .* left intact", sub_line.strip()):
                        body_content = '\n'.join(self._output.splitlines())
                        self.contents.append(OutputContent(self.get_lexer(body_content), body_content))
                        tail_content = '\n'.join(lines[sub_index:])
                        self.contents.append(OutputContent(BashLexer(), tail_content))
                        break
                return

        self.contents.append(OutputContent(BashLexer(), self._stderr))
        body_content = '\n'.join(self._output.splitlines())
        self.contents.append(OutputContent(self.get_lexer(body_content), body_content))

    def _include_parse_output(self):
        if self._stderr.strip() != '':
            self.contents.append(OutputContent(BashLexer(), self._stderr))
        lines = self._output.splitlines()
        for index, line in enumerate(lines):
            if line.strip() == '':
                header_content = '\n'.join(lines[:index])
                self.contents.append(OutputContent(BashLexer(), header_content))
                body_content = '\n'.join(lines[index + 1:])
                self.contents.append(OutputContent(self.get_lexer(body_content), body_content))
                break


    def parse_output(self):
        if self.include and not self.verbose:
            self._include_parse_output()
        if self.verbose:
            self._verbose_parse_output()
        if not self.include and not self.verbose:
            output = self._output
            stderr = self._stderr
            self.contents.append(OutputContent(BashLexer(), stderr))
            self.contents.append(OutputContent(self.get_lexer(output), output))


    @staticmethod
    def get_lexer(content: str) -> RegexLexer:
        try:
            json.loads(content)
            return JsonLexer()
        except JSONDecodeError:
            pass
        return guess_lexer(content)

    def highlight(self):
        for content in self.contents:
            text = content.content
            if isinstance(content.lexer, JsonLexer):
                text = json.dumps(json.loads(text), ensure_ascii=False, indent=4)
            print(highlight(text, content.lexer, TerminalFormatter()), sep=' ', end='')